<!doctype html>
<html lang="en">
  <head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <!-- Bootstrap CSS -->
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css" integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T" crossorigin="anonymous">

    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/open-iconic/1.1.1/font/css/open-iconic-bootstrap.min.css" integrity="sha256-BJ/G+e+y7bQdrYkS2RBTyNfBHpA9IuGaPmf9htub5MQ=" crossorigin="anonymous" />
    
    <link rel='stylesheet' href="{{url_for('.static', filename='style.css')}}">

    

    <title></title>
  </head>
  <body>
    {% include 'dashboard/flashed_messages.html' %}
    {% block layout %}{% endblock %}
    
    {% block scripts %}
        <script src="https://cdn.jsdelivr.net/npm/vue@2.6.10/dist/vue.js"></script>    
        <script src="/dashboard/static/flash_messages.js"></script>
    {% endblock %}
  </body>
</html>