<!DOCTYPE html>


<html>
<head>
        <meta charset="utf-8" />
	<title>Debriefing</title>
	<link rel="stylesheet" href="/static/css/task.css" type="text/css" media="screen" />
</head>
<body>
	<div id="debriefing">
		<h1>Debriefing</h1>

		<div id="debriefingtext">
		<p>Thank you for your participation in our study!  Your anonymous data makes an
		important contribution to our understanding of human learning and memory. </p>

		<p>Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam
		nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat
		volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation
		ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis
		autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie
		consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et
		accumsan et iusto odio dignissim qui blandit praesent luptatum zzril
		delenit augue duis dolore te feugait nulla facilisi. Nam liber tempor cum
		soluta nobis eleifend option congue nihil imperdiet doming id quod mazim
		placerat facer possim assum. Typi non habent claritatem insitam; est usus
		legentis in iis qui facit eorum claritatem. Investigationes demonstraverunt
		lectores legere me lius quod ii legunt saepius. Claritas est etiam
		processus dynamicus, qui sequitur mutationem consuetudium lectorum. Mirum
		est notare quam littera gothica, quam nunc putamus parum claram,
		anteposuerit litterarum formas humanitatis per seacula quarta decima et
		quinta decima. Eodem modo typi, qui nunc nobis videntur parum clari, fiant
		sollemnes in futurum.</p>
		
		<p>If you have any questions about this research, you may contact the
		principal investigator, Principal Investigator.</p>
		</div>

			<p class="cool">Do you agree to the following statement?</p>
			<div id="affirmationbox">
				<p>I feel that I have been adequately debriefed about the nature
					of the study.  The investigator has explained the purposes of the
					research to me, and I feel that any questions I have asked were
					satisfactorily answered.</p>
					<table><tr>
						<td id="affirmative" onClick="javascript:document.forms['accept'].submit()">Yes, I agree.</td>
						<td id="negative" onClick="javascript:document.forms['decline'].submit()">No, please withhold my data. I will contact the experimenter with questions.</td>
					</tr></table> 
					<form id="accept" action="complete" method="post">
						<input type="hidden" name="uniqueid" value="{{ uniqueId }}" />
						<input type="hidden" name="agree" value="true" />
					</form>
					<form id="decline" action="complete" method="post">
						<input type="hidden" name="uniqueid" value="{{ uniqueId }}" />
						<input type="hidden" name="agree" value="false" />
					</form>
			</div>
	</div>
</body>

</html>
