{% extends 'layouts/layout.html' %}

{% block body %}
	<div style="text-align: center;" id="thanks">
		<h1>Booyah, this is an example custom route you can add to your project</h1>
	</div>
{% endblock %}
