{% extends 'layouts/layout.html' %}

{% block head %}
  {{ super() }}
  <script>
    window.opener.location.reload(true);
    window.close();
  </script>
{% endblock %}

{% block body %}
  <h1>Returning to HIT window.</h1>
  <p>You are being redirected back to the HIT window, which should refresh.
    If you do not see the HIT submission form when you return, try refreshing
    the page.</p>
{% endblock %}
