<div id="container-instructions">

    <h1>Instructions</h1>

    <hr />

    <div class="instructions well">

        <p>
            This is the third page of instructions
        </p>

        <p>
            Put some text in this paragraph
        </p>


        <script type="text/javascript">
        //console.log(psiTurk.getInstructionIndicator())
        </script>

    </div>

    <hr />

    <div class="instructionsnav">
        <div class="row">
            <div class="col-xs-2">
                <button type="button" id="prev" value="prev" class="btn btn-primary btn-lg previous">
                <span class="glyphicon glyphicon-arrow-left"></span> Previous
                </button>
            </div>
            <div class="col-xs-8">
            </div>
            <div class="col-xs-2">
                <button type="button" id="next" value="next" class="btn btn-primary btn-lg continue">
                Next <span class="glyphicon glyphicon-arrow-right"></span>
                </button>
            </div>
        </div>
    </div>

</div>
