<div id="container-instructions">

    <h1>Instructions</h1>

    <hr />

    <div class="instructions well">

        <p>
            This is the last page
        </p>
        
        <p>
            You're going to start experiment now
        </p>


        <script type="text/javascript">
        //console.log(psiTurk.getInstructionIndicator())
        </script>

    </div>

    <hr />

    <div class="instructionsnav">
        <div class="row">
            <div class="col-xs-2">
            	<!--  Maybe not previous button on this screen? -->
            	<!--
                <button type="button" id="next" value="next" class="btn btn-primary btn-lg previous">
                <span class="glyphicon glyphicon-arrow-left"></span> Previous
                </button>
            	-->
            </div>
            <div class="col-xs-8">
            	<center>
	                <button type="button" id="next" value="next" class="btn btn-success btn-lg continue">
	                Begin experiment <span class="glyphicon glyphicon-arrow-right"></span>
	                </button>
	            </center>
            </div>
            <div class="col-xs-2">

            </div>
        </div>
    </div>

</div>
