{% extends 'layouts/layout.html' %}
<!--
  The consent.html displays the text of your IRB-approved
  consent form.  Even if you are not required to provide
  a consent form, it is helpful to use this form to
  describe what people can do if an error comes up, etc...

-->
{% block head %}
    {{ super() }}
    <meta charset="utf-8" />
    <title>Psychology Experiment - Informed Consent Form</title>
    <link rel="stylesheet" href="/static/css/bootstrap.min.css" type="text/css" />
    <link rel="stylesheet" href="/static/css/style.css" type="text/css" />
    <link rel="icon" href="/static/favicon.ico" />
    <script type="text/javascript">
        function onexit() {
          self.close(); // no harm, no foul here
        }
    </script>
		<style>
			@media print {
				body * {
					visibility: hidden;
				}
				.legal, .legal * {
					visibility: visible;
				}
				.legal {
					position: absolute;
					left: 0;
					top: 0;
					overflow:visible;
				}
			}
		</style>
    {% endblock %}
{% block body %}
        <div id="container-consent">
            <div id="consent">
                <h1>We need your consent to proceed</h1>
                <hr />
                <div class="legal well">
                    <p>
                    You have been invited to take part in a research study named BLAH BLAH
                    </p>
                    <p>
                    Put your consent form here.  If you are a worker and are viewing this
                    please decline the HIT.  This requester doesn't know what they are doing.
                    </p>

                    <button type="button" class="btn btn-default btn-sm" onClick="window.print();">
                    <span class="glyphicon glyphicon-print"></span> Print a copy of this
                    </button>
                </div>

                <hr />
                <h4>Do you understand and consent to these terms?</h4>
                <br />

                <center>
                    <button type="button" class="btn btn-primary btn-lg" onClick="window.location='/exp?hitId={{ hitid }}&assignmentId={{ assignmentid }}&workerId={{ workerid }}'">
                    <span class="glyphicon glyphicon-ok"></span> I agree
                    </button>
                    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                    <button type="button" class="btn btn-danger btn-lg" onClick="onexit()">
                    <span class="glyphicon glyphicon-ban-circle"></span> No thanks, I do not want to do this HIT
                    </button>
                </center>

            </div>
        </div>
{% endblock %}
