
body {
    padding:0px;
    margin: 0px;
    background-color: white;
    color: black;
    font-weight: 300; 
    font-size: 13pt;
}


/* text styling */
strong {
  font-weight: 600;         
}

.warm {
  color: steelblue;
  text-align: justify;
}

.cool {
  color: #ccccff;
}

h1 {
  margin-top: 0px;
}


/* these are all the same for now, but easy to modify individually */
#container-not-an-ad, #container-error, #container-exp, #container-consent, #container-instructions, #container-questionnaire {
    background: white;
    margin: 30px;
}

/* ad.html  - the ad that people view first */
#adlogo {
    width: 140px;
    padding: 2px;
    border: 1px solid #ccc;
    margin-right: 1em;
}

/* consent.html - the consent form */
#consent {
  margin: 0 auto;
  margin-top: 0;
  width: 800px;
  font-size: 100%;
}

.legal {
  text-align: justify;
  font-size: 11pt;
  line-height: 1.1em;
}

#trial {
  margin: 0 auto;
  width: 800px;
  text-align: center;
}

#query {
  color: darkgray;
}

.question {
  margin-bottom: 30px;
}

/* errors */
#error img {
  padding-right: 10px;
  padding-top: 15px;
}
